﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VMath;

namespace VVVV.DX11.Nodes.Ant.Plugins.File
{
    [PluginInfo(Name ="Player", Category ="File", Version ="Folder")]
    public class FilePlayer : IPluginEvaluate
    {
        [Input("File", DefaultString ="")]
        protected IDiffSpread<string> FInput;

        [Input("Filter", DefaultString = "*.*")]
        protected IDiffSpread<string> FFilter;

        [Input("Up", IsBang = true)]
        protected IDiffSpread<bool> FUp;

        [Input("Down", IsBang = true)]
        protected IDiffSpread<bool> FDown;

        [Input("Random", IsBang = true)]
        protected IDiffSpread<bool> FRandom;

        [Input("Clear", IsBang = true)]
        protected IDiffSpread<bool> FClear;

        [Output("File Output")]
        protected ISpread<string> FOutFile;

        [Output("File Index")]
        protected ISpread<int> FOutFileIndex;

        [Output("BinSize Output")]
        protected ISpread<int> FOutFileBin;

        [Output("Folder Output")]
        protected ISpread<string> FOutFolder;

        [Output("File Taken")]
        protected ISpread<string> FOutComparer;
       
        private bool invalidate = true;

        private List<string>[] FileList;
        private string[] FolderList;

        public void Evaluate(int SpreadMax)
        {
            if (this.FOutComparer.SliceCount != this.FInput.SliceCount)
            {
                invalidate = true;
            }

            if (invalidate)
            {
                this.FileList = new List<string>[FInput.SliceCount];
                this.FolderList = new string[FInput.SliceCount];

                this.FOutFolder.SliceCount = this.FInput.SliceCount;
                this.FOutFile.SliceCount = this.FInput.SliceCount;
                this.FOutFileIndex.SliceCount = this.FInput.SliceCount;
                this.FOutFileBin.SliceCount = this.FInput.SliceCount;
                this.FOutComparer.SliceCount = this.FInput.SliceCount;

                this.invalidate = false;
            }



            if (FInput.IsChanged || FFilter.IsChanged)
            {
    
                for (int i=0; i < FInput.SliceCount; i++)
                {

                    bool valid = System.IO.File.Exists(FInput[i]);

                    if (valid && FInput[i] != FOutComparer[i])
                    {
                        string root = Path.GetDirectoryName(FInput[i]);

                        FOutFolder[i] = root;

                        SearchOption opt = SearchOption.TopDirectoryOnly;
                        //if (FSubDir[i]) { opt = SearchOption.AllDirectories; }
                        List<string> lf = new List<string>(GetFiles(root, FFilter[0], opt));
                        lf.Sort();

                        FileList[i] = new List<string> (lf);
                        FOutFileBin[i] = lf.Count;
                        FOutFileIndex[i] = FileList[i].IndexOf(FInput[i]);
                        FOutComparer[i] = FInput[i];
                    }
                    
                    else if (!valid && FInput[i] != FOutComparer[i])
                    {
                        List<string> lf = new List<string>() { "" };
                        FileList[i] = new List<string> (lf);
                        FOutFolder[i] = "";
                        FOutFileBin[i] = 1;
                        FOutFileIndex[i] = -1;
                        FOutComparer[i] = FInput[i];
                    }
                }
            }

            if (FUp.IsChanged || FDown.IsChanged || FRandom.IsChanged || FClear.IsChanged || FInput.IsChanged)
            {
                for (int i = 0; i < FInput.SliceCount; i++)
                {
                    if (FUp[i]) { FOutFileIndex[i] = (FOutFileIndex[i] + 1) % FOutFileBin[i]; }
                    if (FDown[i]) { FOutFileIndex[i] = VMath.Zmod(FOutFileIndex[i] - 1 , FOutFileBin[i]); }
                    if (FRandom[i]) { Random r = new Random(); FOutFileIndex[i] = (FOutFileIndex[i] + r.Next(0, 9999)) % FOutFileBin[i]; }
                    if (FClear[i]) { FOutFileIndex[i] = -1; }


                    if (FOutFileIndex[i] != -1 && FileList[i] != null)
                    {
                        FOutFile[i] = FileList[i][FOutFileIndex[i]].ToString();
                    }
                    else if (FOutFileIndex[i] == -1 && FileList[i] != null)
                    {
                        FOutFile[i] = "";
                    }
                }
            }
           
        }
        private static string[] GetFiles(string sourceFolder, string filters, System.IO.SearchOption searchOption)
        {
            return filters.Split('|').SelectMany(filter => System.IO.Directory.GetFiles(sourceFolder, filter, searchOption)).ToArray();
        }
    }
}
